require "flush.pl";

################################################  Locking

#
# lock_db - Open and lock the lock-count file, and return an
# event number and the date.
#

sub lock_db {

    if (! open(FH_LOCK, "+<$Lock_count_file")) {
	print "Cannot open file $Lock_count_file!";
	die;
        }

    # Get an exclusive advisory lock; wait if already locked.

    flock(FH_LOCK, 2);
    local ($n) = <FH_LOCK>;
    $n++;
    seek(FH_LOCK, 0, 0);  # go to beginning
    print FH_LOCK "$n\n";
    &flush(FH_LOCK);

    local ($date) = `/bin/date -u`;
    chop($date);

    return ($n,$date);
}  # lock_db

#
# unlock_db - unlock and close the lock-count file.
#

sub unlock_db {
    flock(FH_LOCK, 8); 
    close(FH_LOCK);
}  # unlock_db

################################################  DB insertion and query
#
# We'll use our own field separator, so that we can have ordinary
# text in fields; and our own record separator so that we can have
# multiline records.  These can be changed, but make sure that
# they do not contain any regular expression metacharacters, because
# they are used as ordinary strings and as regular expressions.

$DB_field_sep = " ## ";
# Field separators in fields are replaced with the following.
$DB_not_field_sep = " ### ";
$DB_record_sep = "\n\n-------- end of record --------\n\n";

#
# output_db_record - Given a filename and an array of fields,
# append the fields to the file.  If the field separator occurs in
# any of the fields, replace it with the alternative.
#

sub output_db_record {
    local ($db_filename, @fields) = @_;
    local ($ofs, $ors, $field);

    $ofs = $,;            # save the default output field seperator
    $, = $DB_field_sep;
    $ors = $\;            # save the default output record seperator
    $\ = $DB_record_sep;

    # If any of the fields contains the field separator, replace it.

    foreach $field (@fields) {
	$field =~ s/$DB_field_sep/$DB_not_field_sep/g;
	}

    open(FH, ">>$db_filename");
    print FH @fields;
    close(FH);

    $, = $ofs;            # restore default output separators
    $\ = $ors;
    }  # output_db_record

#
# file_event - Given a filename and a record, lock the database,
# add the event_id and date to the record, output the record, then
# unlock the database.  (Everything is locked, but we're just updating
# one file.  This is inefficient, but ok for this application.)
#

sub file_event {
    local ($file, *record) = @_;
    local ($key, $i, @fields);

    local ($event,$date) = &lock_db;

    $record{"event_id"} = $event;
    $record{"date"} = $date;

    &output_db_record($file, %record);
    &unlock_db;

    return $event;
    }  # file_event

#
# retrieve_events - Given a file and a simple conjucntive query,
# return all records that match.  The query is a list of name-value
# pairs.  The returned records are not split apart; the caller must
# do it something like this:
#
#  @records = &retrieve_events($Report_file,"paper_id",$id,"pc_member",$mem);
#  foreach $record (@records) {
#      %aa = &record_to_assoc_array(*record);
#      $event = $aa{"event_id"};
#

sub retrieve_events {
    local ($file, @query) = @_;

    local ($record, @answers, $name, $value, $ok, $irs, $i, %array);

    if (!open(FH, "<$file")) {
	print "cannot open file $file\n";
	die;
        }

    $irs = $/;            # save the default input record separator
    $/ = $DB_record_sep;

    while ($record = <FH>) {
        $record =~ s/$DB_record_sep//;   # dump rec sep at end of rec
	%array = split($DB_field_sep, $record);
	$ok = 1;
	$i = 0;
	while ($ok && $i < $#query) {
	    $name = $query[$i++];
	    $value = $query[$i++];
	    $ok = ($array{$name} eq $value);
	    }
	if ($ok) {
	    push(@answers, $record);
	    }
	}
    close(FH);
    $/ = $irs;            # restore the default input record seperator

    @answers;
    }  # retrieve_events

#
# record_to_assoc_array - Given a record, make it into an associative array.
#

sub record_to_assoc_array {
    local (*record) = @_;
    return split($DB_field_sep, $record);
}  # record_to_assoc_array

1;  # return true

